@@ -1,5 +1,5 @@
 /*
- *  $Id: setup.h,v 1.7 2010/06/08 05:55:17 schmirl Exp $
+ *  setup.h
  */
  
 #ifndef VDR_STREAMDEV_SETUPCLIENT_H
@@ -17,6 +17,7 @@ struct cStreamdevClientSetup {
 	int  RemotePort;
 	int  Timeout;
 	int  StreamFilters;
+	int  FilterSockBufSize;
 	int  HideMenuEntry;
 	int  LivePriority;
 	int  MinPriority;
@@ -28,15 +29,18 @@ struct cStreamdevClientSetup {
 
 extern cStreamdevClientSetup StreamdevClientSetup;
 
+class cPluginStreamdevClient;
+
 class cStreamdevClientMenuSetupPage: public cMenuSetupPage {
 private:
-	cStreamdevClientSetup m_NewSetup;
+	cPluginStreamdevClient *m_Plugin;
+	cStreamdevClientSetup    m_NewSetup;
 	
 protected:
 	virtual void Store(void);
 
 public:
-	cStreamdevClientMenuSetupPage(void);
+	cStreamdevClientMenuSetupPage(cPluginStreamdevClient *Plugin);
 	virtual ~cStreamdevClientMenuSetupPage();
 };
 
