@@ -1,9 +1,10 @@
 /*
- *  $Id: connectionVTP.c,v 1.31 2010/08/18 10:26:54 schmirl Exp $
+ *  connectionVTP.c
  */
  
 #include "server/connectionVTP.h"
 #include "server/livestreamer.h"
+#include "server/livefilter.h"
 #include "server/suspend.h"
 #include "setup.h"
 
@@ -40,7 +41,9 @@ private:
 	enum eStates { Channel, Event, Title, Subtitle, Description, Vps, Content, Rating,
 	               EndEvent, EndChannel, EndEPG };
 	cConnectionVTP    *m_Client;
+#if APIVERSNUM < 20300
 	cSchedulesLock    *m_SchedulesLock;
+#endif
 	const cSchedules  *m_Schedules;
 	const cSchedule   *m_Schedule;
 	const cEvent      *m_Event;
@@ -57,8 +60,10 @@ public:
 
 cLSTEHandler::cLSTEHandler(cConnectionVTP *Client, const char *Option):
 		m_Client(Client),
+#if APIVERSNUM < 20300
 		m_SchedulesLock(new cSchedulesLock(false, 500)),
 		m_Schedules(cSchedules::Schedules(*m_SchedulesLock)),
+#endif
 		m_Schedule(NULL),
 		m_Event(NULL),
 		m_Errno(0),
@@ -130,11 +135,20 @@ cLSTEHandler::cLSTEHandler(cConnectionVTP *Client, const char *Option):
 					break;
 				}
 			} else if (!m_Schedule) {
+#if APIVERSNUM >= 20300
+				LOCK_CHANNELS_READ;
+				const cChannel* Channel = NULL;
+				if (isnumber(p))
+					Channel = Channels->GetByNumber(strtol(Option, NULL, 10));
+				else
+					Channel = Channels->GetByChannelID(tChannelID::FromString(
+#else
 				cChannel* Channel = NULL;
 				if (isnumber(p))
 					Channel = Channels.GetByNumber(strtol(Option, NULL, 10));
 				else
 					Channel = Channels.GetByChannelID(tChannelID::FromString(
+#endif
 					                                  Option));
 				if (Channel) {
 					m_Schedule = m_Schedules->GetSchedule(Channel->GetChannelID());
@@ -198,7 +212,9 @@ cLSTEHandler::cLSTEHandler(cConnectionVTP *Client, const char *Option):
 
 cLSTEHandler::~cLSTEHandler()
 {
+#if APIVERSNUM < 20300
 	delete m_SchedulesLock;
+#endif
 }
 
 bool cLSTEHandler::Next(bool &Last)
@@ -214,7 +230,12 @@ bool cLSTEHandler::Next(bool &Last)
 	switch (m_State) {
 	case Channel:
 		if (m_Schedule != NULL) {
+#if APIVERSNUM >= 20300
+			LOCK_CHANNELS_READ;
+			const cChannel *channel = Channels->GetByChannelID(m_Schedule->ChannelID(),
+#else
 			cChannel *channel = Channels.GetByChannelID(m_Schedule->ChannelID(),
+#endif
 			                                            true);
 			if (channel != NULL) {
 				m_State = Event;
@@ -370,12 +391,21 @@ cLSTCHandler::cLSTCHandler(cConnectionVTP *Client, const char *Option):
 		m_Errno(0),
 		m_Traverse(false)
 {
+#if APIVERSNUM >= 20300
+	LOCK_CHANNELS_READ;
+	if (!Channels) {
+#else
 	if (!Channels.Lock(false, 500)) {
+#endif
 		m_Errno = 451;
 		m_Error = "Channels are being modified - try again";
 	} else if (*Option) {
 		if (isnumber(Option)) {
+#if APIVERSNUM >= 20300
+			m_Channel = Channels->GetByNumber(strtol(Option, NULL, 10));
+#else
 			m_Channel = Channels.GetByNumber(strtol(Option, NULL, 10));
+#endif
 			if (m_Channel == NULL) {
 				m_Errno = 501;
 				m_Error = cString::sprintf("Channel \"%s\" not defined", Option);
@@ -385,21 +415,35 @@ cLSTCHandler::cLSTCHandler(cConnectionVTP *Client, const char *Option):
 			int i = 1;
 			m_Traverse = true;
 			m_Option = strdup(Option);
+#if APIVERSNUM >= 20300
+			while (i <= Channels->MaxNumber()) {
+				m_Channel = Channels->GetByNumber(i, 1);
+#else
 			while (i <= Channels.MaxNumber()) {
 				m_Channel = Channels.GetByNumber(i, 1);
+#endif
 				if (strcasestr(m_Channel->Name(), Option) != NULL)
 					break;
 				i = m_Channel->Number() + 1;
 			}
 
+#if APIVERSNUM >= 20300
+			if (i > Channels->MaxNumber()) {
+#else
 			if (i > Channels.MaxNumber()) {
+#endif
 				m_Errno = 501;
 				m_Error = cString::sprintf("Channel \"%s\" not defined", Option);
 				return;
 			}
 		}
+#if APIVERSNUM >= 20300
+	} else if (Channels->MaxNumber() >= 1) {
+		m_Channel = Channels->GetByNumber(1, 1);
+#else
 	} else if (Channels.MaxNumber() >= 1) {
 		m_Channel = Channels.GetByNumber(1, 1);
+#endif
 		m_Traverse = true;
 	} else {
 		m_Errno = 550;
@@ -409,7 +453,9 @@ cLSTCHandler::cLSTCHandler(cConnectionVTP *Client, const char *Option):
 
 cLSTCHandler::~cLSTCHandler()
 {
+#if APIVERSNUM < 20300
 	Channels.Unlock();
+#endif
 	if (m_Option != NULL)
 		free(m_Option);
 }
@@ -434,8 +480,14 @@ bool cLSTCHandler::Next(bool &Last)
 	Last = true;
 	if (m_Traverse) {
 		int i = m_Channel->Number() + 1;
+#if APIVERSNUM >= 20300
+		LOCK_CHANNELS_READ;
+		while (i <= Channels->MaxNumber()) {
+			m_Channel = Channels->GetByNumber(i, 1);
+#else
 		while (i <= Channels.MaxNumber()) {
 			m_Channel = Channels.GetByNumber(i, 1);
+#endif
 			if (m_Channel != NULL) {
 				if (m_Option == NULL || strcasestr(m_Channel->Name(), 
 												   m_Option) != NULL)
@@ -447,7 +499,11 @@ bool cLSTCHandler::Next(bool &Last)
 			}
 		}
 
+#if APIVERSNUM >= 20300
+		if (i < Channels->MaxNumber() + 1)
+#else
 		if (i < Channels.MaxNumber() + 1)
+#endif
 			Last = false;
 	}
 
@@ -460,7 +516,11 @@ class cLSTTHandler
 {
 private:
 	cConnectionVTP *m_Client;
+#if APIVERSNUM >= 20300
+	const cTimer   *m_Timer;
+#else
 	cTimer         *m_Timer;
+#endif
 	int             m_Index;
 	int             m_Errno;
 	cString         m_Error;
@@ -478,9 +538,16 @@ cLSTTHandler::cLSTTHandler(cConnectionVTP *Client, const char *Option):
 		m_Errno(0),
 		m_Traverse(false)
 {
+#if APIVERSNUM >= 20300
+	LOCK_TIMERS_READ;
+#endif
 	if (*Option) {
 		if (isnumber(Option)) {
+#if APIVERSNUM >= 20300
+			m_Timer = Timers->Get(strtol(Option, NULL, 10) - 1);
+#else
 			m_Timer = Timers.Get(strtol(Option, NULL, 10) - 1);
+#endif
 			if (m_Timer == NULL) {
 				m_Errno = 501;
 				m_Error = cString::sprintf("Timer \"%s\" not defined", Option);
@@ -489,10 +556,18 @@ cLSTTHandler::cLSTTHandler(cConnectionVTP *Client, const char *Option):
 			m_Errno = 501;
 			m_Error = cString::sprintf("Error in timer number \"%s\"", Option);
 		}
+#if APIVERSNUM >= 20300
+	} else if (Timers->Count()) {
+#else
 	} else if (Timers.Count()) {
+#endif
 		m_Traverse = true;
 		m_Index = 0;
+#if APIVERSNUM >= 20300
+		m_Timer = Timers->Get(m_Index);
+#else
 		m_Timer = Timers.Get(m_Index);
+#endif
 		if (m_Timer == NULL) {
 			m_Errno = 501;
 			m_Error = cString::sprintf("Timer \"%d\" not found", m_Index + 1);
@@ -518,7 +593,12 @@ bool cLSTTHandler::Next(bool &Last)
 
 	bool result;
 	char *buffer;
+#if APIVERSNUM >= 20300
+	LOCK_TIMERS_READ;
+	Last = !m_Traverse || m_Index >= Timers->Count() - 1;
+#else
 	Last = !m_Traverse || m_Index >= Timers.Count() - 1;
+#endif
 	buffer = strdup(*m_Timer->ToText());
 	buffer[strlen(buffer) - 1] = '\0'; // strip \n
 	result = m_Client->Respond(Last ? 250 : -250, "%d %s", m_Timer->Index() + 1,
@@ -526,7 +606,11 @@ bool cLSTTHandler::Next(bool &Last)
 	free(buffer);
 
 	if (m_Traverse && !Last) {
+#if APIVERSNUM >= 20300
+		m_Timer = Timers->Get(++m_Index);
+#else
 		m_Timer = Timers.Get(++m_Index);
+#endif
 		if (m_Timer == NULL) {
 			m_Errno = 501;
 			m_Error = cString::sprintf("Timer \"%d\" not found", m_Index + 1);
@@ -543,7 +627,11 @@ private:
 	enum eStates { Recording, Event, Title, Subtitle, Description, Components, Vps, 
 	               EndRecording };
 	cConnectionVTP *m_Client;
+#if APIVERSNUM >= 20300
+	const cRecording     *m_Recording;
+#else
 	cRecording     *m_Recording;
+#endif
 	const cEvent   *m_Event;
 	int             m_Index;
 	int             m_Errno;
@@ -569,9 +657,16 @@ cLSTRHandler::cLSTRHandler(cConnectionVTP *Client, const char *Option):
 		m_State(Recording),
 		m_CurrentComponent(0)
 {
+#if APIVERSNUM >= 20300
+	LOCK_RECORDINGS_READ;
+#endif
 	if (*Option) {
 		if (isnumber(Option)) {
+#if APIVERSNUM >= 20300
+			m_Recording = Recordings->Get(strtol(Option, NULL, 10) - 1);
+#else
 			m_Recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
+#endif
 			m_Event = m_Recording->Info()->GetEvent();
 			m_Info = true;
 			if (m_Recording == NULL) {
@@ -584,10 +679,18 @@ cLSTRHandler::cLSTRHandler(cConnectionVTP *Client, const char *Option):
 			m_Error = cString::sprintf("Error in Recording number \"%s\"", Option);
 		}
 	} 
+#if APIVERSNUM >= 20300 
+	else if (Recordings->Count()) {
+#else
 	else if (Recordings.Count()) {
+#endif
 		m_Traverse = true;
 		m_Index = 0;
+#if APIVERSNUM >= 20300
+		m_Recording = Recordings->Get(m_Index);
+#else
 		m_Recording = Recordings.Get(m_Index);
+#endif
 		if (m_Recording == NULL) {
 			m_Errno = 501;
 			m_Error = cString::sprintf("Recording \"%d\" not found", m_Index + 1);
@@ -690,11 +793,20 @@ bool cLSTRHandler::Next(bool &Last)
 	}
 	else {
 		bool result;
+#if APIVERSNUM >= 20300
+		LOCK_RECORDINGS_READ;
+		Last = !m_Traverse || m_Index >= Recordings->Count() - 1;
+#else
 		Last = !m_Traverse || m_Index >= Recordings.Count() - 1;
+#endif
 		result = m_Client->Respond(Last ? 250 : -250, "%d %s", m_Recording->Index() + 1, m_Recording->Title(' ', true));
 
 		if (m_Traverse && !Last) {
+#if APIVERSNUM >= 20300
+			m_Recording = Recordings->Get(++m_Index);
+#else
 			m_Recording = Recordings.Get(++m_Index);
+#endif
 			if (m_Recording == NULL) {
 				m_Errno = 501;
 				m_Error = cString::sprintf("Recording \"%d\" not found", m_Index + 1);
@@ -726,7 +838,6 @@ public:
 cConnectionVTP::cConnectionVTP(void): 
 		cServerConnection("VTP"),
 		m_LiveSocket(NULL),
-		m_LiveStreamer(NULL),
 		m_FilterSocket(NULL),
 		m_FilterStreamer(NULL),
 		m_RecSocket(NULL),
@@ -753,7 +864,8 @@ cConnectionVTP::~cConnectionVTP()
 {
 	if (m_LastCommand != NULL) 
 		free(m_LastCommand);
-	delete m_LiveStreamer;
+	if (Streamer())
+		Streamer()->Stop();
 	delete m_LiveSocket;
 	delete m_RecSocket;
 	delete m_FilterStreamer;
@@ -768,8 +880,8 @@ cConnectionVTP::~cConnectionVTP()
 
 bool cConnectionVTP::Abort(void) const
 {
-	return !IsOpen() || (m_LiveStreamer && m_LiveStreamer->Abort()) ||
-		(m_FilterStreamer && m_FilterStreamer->Abort());
+	return !IsOpen() || (Streamer() && Streamer()->Abort()) ||
+		(!Streamer() && m_FilterStreamer && m_FilterStreamer->Abort());
 }
 
 void cConnectionVTP::Welcome(void) 
@@ -785,13 +897,13 @@ void cConnectionVTP::Reject(void)
 
 void cConnectionVTP::Detach(void) 
 {
-	if (m_LiveStreamer) m_LiveStreamer->Detach();
 	if (m_FilterStreamer) m_FilterStreamer->Detach();
+	if (m_LiveSocket && Streamer()) ((cStreamdevLiveStreamer *) Streamer())->Detach();
 }
 
 void cConnectionVTP::Attach(void) 
 {
-	if (m_LiveStreamer) m_LiveStreamer->Attach();
+	if (m_LiveSocket && Streamer()) ((cStreamdevLiveStreamer *) Streamer())->Attach();
 	if (m_FilterStreamer) m_FilterStreamer->Attach();
 }
 
@@ -871,10 +983,12 @@ bool cConnectionVTP::CmdCAPS(char *Opts)
 		return Respond(220, "Capability \"%s\" accepted", Opts);
 	}
 
+#ifdef STREAMDEV_PS
 	if (strcasecmp(Opts, "PS") == 0) {
 		m_StreamType = stPS;
 		return Respond(220, "Capability \"%s\" accepted", Opts);
 	}
+#endif
 
 	if (strcasecmp(Opts, "PES") == 0) {
 		m_StreamType = stPES;
@@ -933,7 +1047,7 @@ bool cConnectionVTP::CmdPROV(char *Opts)
 
 	LOOP_PREVENTION(chan);
 
-	if (ProvidesChannel(chan, prio)) {
+	if (cStreamdevLiveStreamer::ProvidesChannel(chan, prio)) {
 		m_TuneChannel = chan;
 		m_TunePriority = prio;
 		return Respond(220, "Channel available");
@@ -942,7 +1056,7 @@ bool cConnectionVTP::CmdPROV(char *Opts)
 	// so get our own receiver temporarily out of the way
 	if (m_ClientVersion == 0) {
 		Detach();
-		bool provided = ProvidesChannel(chan, prio);
+		bool provided = cStreamdevLiveStreamer::ProvidesChannel(chan, prio);
 		Attach();
 		if (provided) {
 			m_TuneChannel = chan;
@@ -1011,14 +1125,13 @@ bool cConnectionVTP::CmdPORT(char *Opts)
 		if(!m_FilterStreamer)
 			m_FilterStreamer = new cStreamdevFilterStreamer;
 		m_FilterStreamer->Start(m_FilterSocket);
-		m_FilterStreamer->Activate(true);
 
 		return Respond(220, "Port command ok, data connection opened");
 		break;
 
 	case siLive:
-		if(m_LiveSocket && m_LiveStreamer)
-			m_LiveStreamer->Stop();
+		if(m_LiveSocket && Streamer())
+			Streamer()->Stop();
 		delete m_LiveSocket;
 
 		m_LiveSocket = new cTBSocket(SOCK_STREAM);
@@ -1031,8 +1144,8 @@ bool cConnectionVTP::CmdPORT(char *Opts)
 
 		if (!m_LiveSocket->SetDSCP())
 			LOG_ERROR_STR("unable to set DSCP sockopt");
-		if (m_LiveStreamer)
-			m_LiveStreamer->Start(m_LiveSocket);
+		if (Streamer())
+			Streamer()->Start(m_LiveSocket);
 
 		return Respond(220, "Port command ok, data connection opened");
 		break;
@@ -1115,21 +1228,19 @@ bool cConnectionVTP::CmdTUNE(char *Opts)
 	if (chan != m_TuneChannel) {
 		isyslog("streamdev-server TUNE %s: Priority unknown - using 0", Opts);
 		prio = 0;
-		if (!ProvidesChannel(chan, prio))
+		if (!cStreamdevLiveStreamer::ProvidesChannel(chan, prio))
 			return Respond(560, "Channel not available (ProvidesChannel)");
 	}
-	if ((dev = GetDevice(chan, prio)) == NULL)
-		return Respond(560, "Channel not available (GetDevice)");
 
-	if (!dev->SwitchChannel(chan, false))
-		return Respond(560, "Channel not available (SwitchChannel)");
+	cStreamdevLiveStreamer* liveStreamer = new cStreamdevLiveStreamer(this, chan, prio, m_StreamType);
+	if ((dev = liveStreamer->GetDevice()) == NULL) {
+		delete liveStreamer;
+		return Respond(560, "Channel not available (SwitchDevice)");
+	}
+	SetStreamer(liveStreamer);
 
-	delete m_LiveStreamer;
-	m_LiveStreamer = new cStreamdevLiveStreamer(prio, this);
-	m_LiveStreamer->SetChannel(chan, m_StreamType);
-	m_LiveStreamer->SetDevice(dev);
 	if(m_LiveSocket)
-		m_LiveStreamer->Start(m_LiveSocket);
+		liveStreamer->Start(m_LiveSocket);
 	
 	if(m_FiltersSupport) {
 		if(!m_FilterStreamer)
@@ -1145,12 +1256,17 @@ bool cConnectionVTP::CmdPLAY(char *Opts)
 {
 	if (*Opts) {
 		if (isnumber(Opts)) {
+#if APIVERSNUM >= 20300
+			LOCK_RECORDINGS_READ;
+			const cRecording *recording = Recordings->Get(strtol(Opts, NULL, 10) - 1);
+#else
 			cRecording *recording = Recordings.Get(strtol(Opts, NULL, 10) - 1);
+#endif
 			if (recording) {
 				if (m_RecPlayer) {
 					delete m_RecPlayer;
 				}
-				m_RecPlayer = new RecPlayer(recording);
+				m_RecPlayer = new RecPlayer(recording->FileName());
 				return Respond(220, "%llu (Bytes), %u (Frames)", (long long unsigned int) m_RecPlayer->getLengthBytes(), (unsigned int) m_RecPlayer->getLengthFrames());
 			}
 			else {
@@ -1175,8 +1291,8 @@ bool cConnectionVTP::CmdPRIO(char *Opts)
 	if (end == Opts || (*end != '\0' && *end != ' '))
 		return Respond(500, "Use: PRIO Priority");
 
-	if (m_LiveStreamer) {
-		m_LiveStreamer->SetPriority(prio);
+	if (Streamer()) {
+		((cStreamdevLiveStreamer*) Streamer())->SetPriority(prio);
 		return Respond(220, "Priority changed to %d", prio);
 	}
 	return Respond(550, "Priority not applicable");
@@ -1184,11 +1300,11 @@ bool cConnectionVTP::CmdPRIO(char *Opts)
 
 bool cConnectionVTP::CmdSGNL(char *Opts) 
 {
-	if (m_LiveStreamer) {
+	if (Streamer()) {
 		int devnum = -1;
 		int signal = -1;
 		int quality = -1;
-		m_LiveStreamer->GetSignal(&devnum, &signal, &quality);
+		((cStreamdevLiveStreamer*) Streamer())->GetSignal(&devnum, &signal, &quality);
 		return Respond(220, "%d %d:%d", devnum, signal, quality);
 	}
 	return Respond(550, "Signal not applicable");
@@ -1203,7 +1319,7 @@ bool cConnectionVTP::CmdADDP(char *Opts)
 	if (end == Opts || (*end != '\0' && *end != ' '))
 		return Respond(500, "Use: ADDP Pid");
 
-	return m_LiveStreamer && m_LiveStreamer->SetPid(pid, true)
+	return Streamer() && ((cStreamdevLiveStreamer*) Streamer())->SetPid(pid, true)
 			? Respond(220, "Pid %d available", pid)
 			: Respond(560, "Pid %d not available", pid);
 }
@@ -1217,7 +1333,7 @@ bool cConnectionVTP::CmdDELP(char *Opts)
 	if (end == Opts || (*end != '\0' && *end != ' '))
 		return Respond(500, "Use: DELP Pid");
 
-	return m_LiveStreamer && m_LiveStreamer->SetPid(pid, false)
+	return Streamer() && ((cStreamdevLiveStreamer*) Streamer())->SetPid(pid, false)
 			? Respond(220, "Pid %d stopped", pid)
 			: Respond(560, "Pid %d not transferring", pid);
 }
@@ -1282,8 +1398,8 @@ bool cConnectionVTP::CmdABRT(char *Opts)
 
 	switch (id) {
 	case siLive: 
-		if (m_LiveStreamer)
-			m_LiveStreamer->Stop();
+		if (Streamer())
+			Streamer()->Stop();
 		DELETENULL(m_LiveSocket);
 		break;
 	case siLiveFilter:
@@ -1377,7 +1493,11 @@ bool cConnectionVTP::CmdSTAT(const char *Option)
 	if (*Option) {
 		if (strcasecmp(Option, "DISK") == 0) {
 			int FreeMB, UsedMB;
+#if APIVERSNUM < 20102
 			int Percent = VideoDiskSpace(&FreeMB, &UsedMB);
+#else
+			int Percent = cVideoDirectory::VideoDiskSpace(&FreeMB, &UsedMB);
+#endif
 			Reply(250, "%dMB %dMB %d%%", FreeMB + UsedMB, FreeMB, Percent);
 		}
 		else if (strcasecmp(Option, "NAME") == 0) {
@@ -1387,10 +1507,17 @@ bool cConnectionVTP::CmdSTAT(const char *Option)
 			Reply(250, "VDR: %s | Streamdev: %s", VDRVERSION, VERSION);
 		}
 		else if (strcasecmp(Option, "RECORDS") == 0) {
+#if APIVERSNUM >= 20300
+			LOCK_RECORDINGS_WRITE;
+			Recordings->Sort();
+			if (Recordings) {
+				cRecording *recording = Recordings->Last();
+#else
 			bool recordings = Recordings.Load();
 			Recordings.Sort();
 			if (recordings) {
 				cRecording *recording = Recordings.Last();
+#endif
 				Reply(250, "%d", recording->Index() + 1);
 			}
 			else {
@@ -1398,10 +1525,20 @@ bool cConnectionVTP::CmdSTAT(const char *Option)
 			}
 		}
 		else if (strcasecmp(Option, "CHANNELS") == 0) {
+#if APIVERSNUM >= 20300 
+			LOCK_CHANNELS_READ;
+			Reply(250, "%d", Channels->MaxNumber());
+#else
 			Reply(250, "%d", Channels.MaxNumber());
+#endif
 		}
 		else if (strcasecmp(Option, "TIMERS") == 0) {
+#if APIVERSNUM >= 20300 
+			LOCK_TIMERS_READ;
+			Reply(250, "%d", Timers->Count());
+#else
 			Reply(250, "%d", Timers.Count());
+#endif
 		}
 		else if (strcasecmp(Option, "CHARSET") == 0) {
 			Reply(250, "%s", cCharSetConv::SystemCharacterTable());
@@ -1429,7 +1566,12 @@ bool cConnectionVTP::CmdMODT(const char *Option)
 		int n = strtol(Option, &tail, 10);
 		if (tail && tail != Option) {
 			tail = skipspace(tail);
+#if APIVERSNUM >= 20300
+			LOCK_TIMERS_WRITE;
+			cTimer *timer = Timers->Get(n - 1);
+#else
 			cTimer *timer = Timers.Get(n - 1);
+#endif
 			if (timer) {
 				cTimer t = *timer;
 				if (strcasecmp(tail, "ON") == 0)
@@ -1441,7 +1583,11 @@ bool cConnectionVTP::CmdMODT(const char *Option)
 					EXIT_WRAPPER();
 				}
 				*timer = t;
+#if APIVERSNUM >= 20300
+				Timers->SetModified();
+#else
 				Timers.SetModified();
+#endif
 				isyslog("timer %s modified (%s)", *timer->ToDescr(), 
 				        timer->HasFlags(tfActive) ? "active" : "inactive");
 				Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
@@ -1460,10 +1606,18 @@ bool cConnectionVTP::CmdNEWT(const char *Option)
 	if (*Option) {
 		cTimer *timer = new cTimer;
 		if (timer->Parse(Option)) {
+#if APIVERSNUM >= 20300
+			LOCK_TIMERS_WRITE;
+			cTimer *t = Timers->GetTimer(timer);
+			if (!t) {
+				Timers->Add(timer);
+				Timers->SetModified();
+#else
 			cTimer *t = Timers.GetTimer(timer);
 			if (!t) {
 				Timers.Add(timer);
 				Timers.SetModified();
+#endif
 				isyslog("timer %s added", *timer->ToDescr());
 				Reply(250, "%d %s", timer->Index() + 1, *timer->ToText());
 				EXIT_WRAPPER();
@@ -1508,21 +1662,39 @@ bool cConnectionVTP::CmdDELT(const char *Option)
 			}
 		}
 
+#if APIVERSNUM >= 20300
+		LOCK_TIMERS_WRITE;
+		cTimer *Timer = Timers->Get(number);
+			if (Timer) {
+			if (Timer->Recording()) {
+				if (force) {
+					if (!Timer->Remote()) {
+						Timer->Skip();
+						cRecordControls::Process(Timers, time(NULL));
+					}
+#else
 		cTimer *timer = Timers.Get(number);
 			if (timer) {
 			if (timer->Recording()) {
 				if (force) {
 					timer->Skip();
 					cRecordControls::Process(time(NULL));
+#endif
 				}
 				else {
 					Reply(550, "Timer \"%i\" is recording", number);
 					EXIT_WRAPPER();
 				}
 			}
+#if APIVERSNUM >= 20300
+					isyslog("deleting timer %s", *Timer->ToDescr());
+					Timers->Del(Timer);
+					Timers->SetModified();
+#else
 					isyslog("deleting timer %s", *timer->ToDescr());
 					Timers.Del(timer);
 					Timers.SetModified();
+#endif
 			Reply(250, "Timer \"%i\" deleted", number);
 				} else
 			Reply(501, "Timer \"%i\" not defined", number);
@@ -1534,7 +1706,12 @@ bool cConnectionVTP::CmdDELT(const char *Option)
 bool cConnectionVTP::CmdNEXT(const char *Option)
 {
 	INIT_WRAPPER();
+#if APIVERSNUM >= 20300
+	LOCK_TIMERS_READ;
+	const cTimer *t = Timers->GetNextActiveTimer();
+#else
 	cTimer *t = Timers.GetNextActiveTimer();
+#endif
 	if (t) {
 		time_t Start = t->StartTime();
 		int Number = t->Index() + 1;
@@ -1558,12 +1735,23 @@ bool cConnectionVTP::CmdNEWC(const char *Option)
 	if (*Option) {
 		cChannel ch;
 		if (ch.Parse(Option)) {
+#if APIVERSNUM >= 20300
+			LOCK_CHANNELS_WRITE;
+			if (Channels->HasUniqueChannelID(&ch)) {
+#else
 			if (Channels.HasUniqueChannelID(&ch)) {
+#endif
 				cChannel *channel = new cChannel;
 				*channel = ch;
+#if APIVERSNUM >= 20300 
+				Channels->Add(channel);
+				Channels->ReNumber();
+				Channels->SetModified();
+#else
 				Channels.Add(channel);
 				Channels.ReNumber();
 				Channels.SetModified(true);
+#endif
 				isyslog("new channel %d %s", channel->Number(), *channel->ToText());
 				Reply(250, "%d %s", channel->Number(), *channel->ToText());
 			}
@@ -1589,15 +1777,28 @@ bool cConnectionVTP::CmdMODC(const char *Option)
 		int n = strtol(Option, &tail, 10);
 		if (tail && tail != Option) {
 			tail = skipspace(tail);
+#if APIVERSNUM >= 20300         
+			LOCK_CHANNELS_WRITE;
+			Channels->SetExplicitModify();
+				cChannel *channel = Channels->GetByNumber(n);
+#else
 			if (!Channels.BeingEdited()) {
 				cChannel *channel = Channels.GetByNumber(n);
+#endif
 				if (channel) {
 					cChannel ch;
 					if (ch.Parse(tail)) {
+#if APIVERSNUM >= 20300         
+						if (Channels->HasUniqueChannelID(&ch, channel)) {
+							*channel = ch;
+							Channels->ReNumber();
+							Channels->SetModified();
+#else
 						if (Channels.HasUniqueChannelID(&ch, channel)) {
 							*channel = ch;
 							Channels.ReNumber();
 							Channels.SetModified(true);
+#endif
 							isyslog("modifed channel %d %s", channel->Number(), *channel->ToText());
 							Reply(250, "%d %s", channel->Number(), *channel->ToText());
 						}
@@ -1612,10 +1813,12 @@ bool cConnectionVTP::CmdMODC(const char *Option)
 				else {
 					Reply(501, "Channel \"%d\" not defined", n);
 				}
+#if APIVERSNUM < 20300
 			}
 			else {
 				Reply(550, "Channels are being edited - try again later");
 			}
+#endif
 		}
 		else {
 			Reply(501, "Error in channel number");
@@ -1631,7 +1834,14 @@ bool cConnectionVTP::CmdMOVC(const char *Option)
 {
 	INIT_WRAPPER();
 	if (*Option) {
+#if APIVERSNUM >= 20300
+		LOCK_CHANNELS_WRITE;
+		Channels->SetExplicitModify();
+//		LOCK_TIMERS_WRITE;
+//		Timers->SetExplicitModify();
+#else
 		if (!Channels.BeingEdited() && !Timers.BeingEdited()) {
+#endif
 			char *tail;
 			int From = strtol(Option, &tail, 10);
 			if (tail && tail != Option) {
@@ -1639,30 +1849,51 @@ bool cConnectionVTP::CmdMOVC(const char *Option)
 				if (tail && tail != Option) {
 					int To = strtol(tail, NULL, 10);
 					int CurrentChannelNr = cDevice::CurrentChannel();
+#if APIVERSNUM >= 20300
+					cChannel *CurrentChannel = Channels->GetByNumber(CurrentChannelNr);
+					cChannel *FromChannel = Channels->GetByNumber(From);
+					if (FromChannel) {
+						cChannel *ToChannel = Channels->GetByNumber(To);
+#else
 					cChannel *CurrentChannel = Channels.GetByNumber(CurrentChannelNr);
 					cChannel *FromChannel = Channels.GetByNumber(From);
 					if (FromChannel) {
 						cChannel *ToChannel = Channels.GetByNumber(To);
+#endif
 						if (ToChannel) {
 							int FromNumber = FromChannel->Number();
 							int ToNumber = ToChannel->Number();
 							if (FromNumber != ToNumber) {
+#if APIVERSNUM >= 20300
+								Channels->Move(FromChannel, ToChannel);
+								Channels->ReNumber();
+								Channels->SetModified();
+#else
 								Channels.Move(FromChannel, ToChannel);
 								Channels.ReNumber();
 								Channels.SetModified(true);
+#endif
 								if (CurrentChannel && CurrentChannel->Number() != CurrentChannelNr) {
 									if (!cDevice::PrimaryDevice()->Replaying() || cDevice::PrimaryDevice()->Transferring()) {
+#if APIVERSNUM >= 20300
+										Channels->SwitchTo(CurrentChannel->Number());
+#else
 										Channels.SwitchTo(CurrentChannel->Number());
+#endif
 									}
 									else {
+#if APIVERSNUM >= 20302
+										cDevice::SetCurrentChannel(CurrentChannel->Number());
+#else
 										cDevice::SetCurrentChannel(CurrentChannel);
+#endif
 									}
 								}
 								isyslog("channel %d moved to %d", FromNumber, ToNumber);
 								Reply(250,"Channel \"%d\" moved to \"%d\"", From, To);
 							}
 							else {
-								Reply(501, "Can't move channel to same postion");
+								Reply(501, "Can't move channel to same position");
 							}
 						}
 						else {
@@ -1680,10 +1911,12 @@ bool cConnectionVTP::CmdMOVC(const char *Option)
 			else {
 				Reply(501, "Error in channel number");
 			}
+#if APIVERSNUM < 20300
 		}
 		else {
 			Reply(550, "Channels or timers are being edited - try again later");
 		}
+#endif
 	}
 	else {
 		Reply(501, "Missing channel number");
@@ -1696,42 +1929,80 @@ bool cConnectionVTP::CmdDELC(const char *Option)
 	INIT_WRAPPER();
 	if (*Option) {
 		if (isnumber(Option)) {
+#if APIVERSNUM >= 20300
+			LOCK_CHANNELS_WRITE;
+			Channels->SetExplicitModify();
+			cChannel *channel = Channels->GetByNumber(strtol(Option, NULL, 10));
+#else
 			if (!Channels.BeingEdited()) {
 				cChannel *channel = Channels.GetByNumber(strtol(Option, NULL, 10));
+#endif
 				if (channel) {
+#if APIVERSNUM >= 20300
+					LOCK_TIMERS_READ;
+					for (const cTimer *timer = Timers->First(); timer; timer = Timers->Next(timer)) {
+#else
 					for (cTimer *timer = Timers.First(); timer; timer = Timers.Next(timer)) {
+#endif
 						if (timer->Channel() == channel) {
 							Reply(550, "Channel \"%s\" is in use by timer %d", Option, timer->Index() + 1);
 							return false;
 						}
 					}
 					int CurrentChannelNr = cDevice::CurrentChannel();
+#if APIVERSNUM >= 20300
+					cChannel *CurrentChannel = Channels->GetByNumber(CurrentChannelNr);
+#else
 					cChannel *CurrentChannel = Channels.GetByNumber(CurrentChannelNr);
+#endif
 					if (CurrentChannel && channel == CurrentChannel) {
+#if APIVERSNUM >= 20300
+						int n = Channels->GetNextNormal(CurrentChannel->Index());
+						if (n < 0)
+							n = Channels->GetPrevNormal(CurrentChannel->Index());
+						CurrentChannel = Channels->Get(n);
+#else
 						int n = Channels.GetNextNormal(CurrentChannel->Index());
 						if (n < 0)
 							n = Channels.GetPrevNormal(CurrentChannel->Index());
 						CurrentChannel = Channels.Get(n);
+#endif
 						CurrentChannelNr = 0; // triggers channel switch below
 					}
+#if APIVERSNUM >= 20300
+					Channels->Del(channel);
+					Channels->ReNumber();
+					Channels->SetModified();
+#else
 					Channels.Del(channel);
 					Channels.ReNumber();
 					Channels.SetModified(true);
+#endif
 					isyslog("channel %s deleted", Option);
 					if (CurrentChannel && CurrentChannel->Number() != CurrentChannelNr) {
 						if (!cDevice::PrimaryDevice()->Replaying() || cDevice::PrimaryDevice()->Transferring())
+#if APIVERSNUM >= 20300
+							Channels->SwitchTo(CurrentChannel->Number());
+#else
 							Channels.SwitchTo(CurrentChannel->Number());
+#endif
 						else
+#if APIVERSNUM >= 20302
+							cDevice::SetCurrentChannel(CurrentChannel->Number());
+#else
 							cDevice::SetCurrentChannel(CurrentChannel);
+#endif
 					}
 					Reply(250, "Channel \"%s\" deleted", Option);
 				}
 				else
 					Reply(501, "Channel \"%s\" not defined", Option);
 			}
+#if APIVERSNUM < 20300
 			else
 				Reply(550, "Channels are being edited - try again later");
 		}
+#endif
 		else
 			Reply(501, "Error in channel number \"%s\"", Option);
 	}
@@ -1746,13 +2017,22 @@ bool cConnectionVTP::CmdDELR(const char *Option)
 	INIT_WRAPPER();
 	if (*Option) {
 		if (isnumber(Option)) {
+#if APIVERSNUM >= 20300
+			LOCK_RECORDINGS_WRITE;
+			cRecording *recording = Recordings->Get(strtol(Option, NULL, 10) - 1);
+#else
 			cRecording *recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
+#endif
 			if (recording) {
 				cRecordControl *rc = cRecordControls::GetRecordControl(recording->FileName());
 				if (!rc) {
 					if (recording->Delete()) {
 						Reply(250, "Recording \"%s\" deleted", Option);
+#if APIVERSNUM >= 20300
+						Recordings->DelByName(recording->FileName());
+#else
 						::Recordings.DelByName(recording->FileName());
+#endif
 					}
 					else
 						Reply(554, "Error while deleting recording!");
@@ -1761,7 +2041,11 @@ bool cConnectionVTP::CmdDELR(const char *Option)
 					Reply(550, "Recording \"%s\" is in use by timer %d", Option, rc->Timer()->Index() + 1);
 			}
 			else
+#if APIVERSNUM >= 20300
+				Reply(550, "Recording \"%s\" not found%s", Option, Recordings->Count() ? "" : " (use LSTR before deleting)");
+#else
 				Reply(550, "Recording \"%s\" not found%s", Option, Recordings.Count() ? "" : " (use LSTR before deleting)");
+#endif
 		}
 		else
 			Reply(501, "Error in recording number \"%s\"", Option);
@@ -1815,7 +2099,11 @@ bool cConnectionVTP::Respond(int Code, const char *Message, ...)
 {
 	va_list ap;
 	va_start(ap, Message);
+#if APIVERSNUM >= 10728
+	cString str = cString::vsprintf(Message, ap);
+#else
 	cString str = cString::sprintf(Message, ap);
+#endif
 	va_end(ap);
 
 	if (Code >= 0 && m_LastCommand != NULL) {
@@ -1828,13 +2116,13 @@ bool cConnectionVTP::Respond(int Code, const char *Message, ...)
 				Code < 0 ? '-' : ' ', *str);
 }
 
-cString cConnectionVTP::ToText() const
+cString cConnectionVTP::ToText(char Delimiter) const
 {
-	cString str = cServerConnection::ToText();
-	if (m_LiveStreamer)
-		return cString::sprintf("%s\t%s", *str, *m_LiveStreamer->ToText());
+	cString str = cServerConnection::ToText(Delimiter);
+	if (Streamer())
+		return cString::sprintf("%s%c%s", *str, Delimiter, *Streamer()->ToText());
 	else if (m_RecPlayer)
-		return cString::sprintf("%s\t%s", *str, m_RecPlayer->getCurrentRecording()->Name());
+		return cString::sprintf("%s%c%s", *str, Delimiter, m_RecPlayer->getCurrentRecording()->Name());
 	else
 		return str;
 }
