@@ -8,33 +8,41 @@
 #include "remux/ts2es.h"
 #include "remux/extern.h"
 
-#include <vdr/ringbuffer.h>
+#include <vdr/transfer.h>
 
 #include "server/livestreamer.h"
-#include "server/livefilter.h"
+#include "server/setup.h"
 #include "common.h"
 
 using namespace Streamdev;
 
+// device occupied timeout to prevent VDR main loop to immediately switch back
+// when streamdev switched the live TV channel.
+// Note that there is still a gap between the GetDevice() and SetOccupied()
+// calls where the VDR main loop could strike
+#define STREAMDEVTUNETIMEOUT 5
+
 // --- cStreamdevLiveReceiver -------------------------------------------------
 
 class cStreamdevLiveReceiver: public cReceiver {
 	friend class cStreamdevStreamer;
 
 private:
-	cStreamdevStreamer *m_Streamer;
+	cStreamdevLiveStreamer *m_Streamer;
 
 protected:
-	virtual void Activate(bool On);
+#if APIVERSNUM >= 20300
+	virtual void Receive(const uchar *Data, int Length);
+#else
 	virtual void Receive(uchar *Data, int Length);
+#endif
 
 public:
-	cStreamdevLiveReceiver(cStreamdevStreamer *Streamer, const cChannel *Channel, int Priority, const int *Pids);
+	cStreamdevLiveReceiver(cStreamdevLiveStreamer *Streamer, const cChannel *Channel, int Priority, const int *Pids);
 	virtual ~cStreamdevLiveReceiver();
 };
 
-cStreamdevLiveReceiver::cStreamdevLiveReceiver(cStreamdevStreamer *Streamer, const cChannel *Channel, 
-                                               int Priority, const int *Pids):
+cStreamdevLiveReceiver::cStreamdevLiveReceiver(cStreamdevLiveStreamer *Streamer, const cChannel *Channel, int Priority, const int *Pids):
 		cReceiver(Channel, Priority),
 		m_Streamer(Streamer)
 {
@@ -43,22 +51,18 @@ cStreamdevLiveReceiver::cStreamdevLiveReceiver(cStreamdevStreamer *Streamer, con
 		AddPids(Pids);
 }
 
-cStreamdevLiveReceiver::~cStreamdevLiveReceiver() 
+cStreamdevLiveReceiver::~cStreamdevLiveReceiver()
 {
 	Dprintf("Killing live receiver\n");
 	Detach();
 }
 
+#if APIVERSNUM >= 20300
+void cStreamdevLiveReceiver::Receive(const uchar *Data, int Length) {
+#else
 void cStreamdevLiveReceiver::Receive(uchar *Data, int Length) {
-	int p = m_Streamer->Receive(Data, Length);
-	if (p != Length)
-		m_Streamer->ReportOverflow(Length - p);
-}
-
-inline void cStreamdevLiveReceiver::Activate(bool On) 
-{ 
-	Dprintf("LiveReceiver->Activate(%d)\n", On);
-	m_Streamer->Activate(On); 
+#endif
+	m_Streamer->Receive(Data, Length);
 }
 
 // --- cStreamdevPatFilter ----------------------------------------------------
@@ -119,14 +123,14 @@ cStreamdevPatFilter::cStreamdevPatFilter(cStreamdevLiveStreamer *Streamer, const
 }
 
 static const char * const psStreamTypes[] = {
-	"UNKNOWN", 
-	"ISO/IEC 11172 Video", 
-	"ISO/IEC 13818-2 Video", 
-	"ISO/IEC 11172 Audio", 
+	"UNKNOWN",
+	"ISO/IEC 11172 Video",
+	"ISO/IEC 13818-2 Video",
+	"ISO/IEC 11172 Audio",
 	"ISO/IEC 13818-3 Audio",
-	"ISO/IEC 13818-1 Privete sections", 
+	"ISO/IEC 13818-1 Privete sections",
 	"ISO/IEC 13818-1 Private PES data",
-	"ISO/IEC 13512 MHEG", 
+	"ISO/IEC 13512 MHEG",
 	"ISO/IEC 13818-1 Annex A DSM CC",
 	"0x09",
 	"ISO/IEC 13818-6 Multiprotocol encapsulation",
@@ -139,6 +143,8 @@ static const char * const psStreamTypes[] = {
 	"ISO/IEC 14496-3 Audio with LATM transport syntax",
 	"0x12", "0x13", "0x14", "0x15", "0x16", "0x17", "0x18", "0x19", "0x1a",
 	"ISO/IEC 14496-10 Video (MPEG-4 part 10/AVC, aka H.264)",
+	"0x1c", "0x1d", "0x1e", "0x1f", "0x20", "0x21", "0x22","0x23",
+	"HEVC aka H.265",
 	"",
 };
 
@@ -146,7 +152,7 @@ int cStreamdevPatFilter::GetPid(SI::PMT::Stream& stream)
 {
 	SI::Descriptor *d;
 
-	if (!stream.getPid()) 
+	if (!stream.getPid())
 		return 0;
 
 	switch (stream.getStreamType()) {
@@ -155,7 +161,7 @@ int cStreamdevPatFilter::GetPid(SI::PMT::Stream& stream)
 	case 0x03: // ISO/IEC 11172 Audio
 	case 0x04: // ISO/IEC 13818-3 Audio
 #if 0
-	case 0x07: // ISO/IEC 13512 MHEG 
+	case 0x07: // ISO/IEC 13512 MHEG
 	case 0x08: // ISO/IEC 13818-1 Annex A  DSM CC
 	case 0x0a: // ISO/IEC 13818-6 Multiprotocol encapsulation
 	case 0x0b: // ISO/IEC 13818-6 DSM-CC U-N Messages
@@ -167,6 +173,7 @@ int cStreamdevPatFilter::GetPid(SI::PMT::Stream& stream)
 	case 0x10: // ISO/IEC 14496-2 Visual (MPEG-4)
 	case 0x11: // ISO/IEC 14496-3 Audio with LATM transport syntax
 	case 0x1b: // ISO/IEC 14496-10 Video (MPEG-4 part 10/AVC, aka H.264)
+	case 0x24: // HEVC aka H.265
 		Dprintf("cStreamdevPatFilter PMT scanner adding PID %d (%s)\n",
 			stream.getPid(), psStreamTypes[stream.getStreamType()]);
 		return stream.getPid();
@@ -199,9 +206,9 @@ int cStreamdevPatFilter::GetPid(SI::PMT::Stream& stream)
 		}
 		break;
 	default:
-		/* This following section handles all the cases where the audio track 
+		/* This following section handles all the cases where the audio track
 		 * info is stored in PMT user info with stream id >= 0x80
-		 * we check the registration format identifier to see if it 
+		 * we check the registration format identifier to see if it
 		 * holds "AC-3"
 		 */
 		if (stream.getStreamType() >= 0x80) {
@@ -217,22 +224,22 @@ int cStreamdevPatFilter::GetPid(SI::PMT::Stream& stream)
 						    rawdata[2] == 'A' && rawdata[3] == 'C' &&
 						    rawdata[4] == '-' && rawdata[5] == '3') {
 							isyslog("cStreamdevPatFilter PMT scanner:"
-								"Adding pid %d (type 0x%x) RegDesc len %d (%c%c%c%c)", 
-								stream.getPid(), stream.getStreamType(), 
-								d->getLength(), rawdata[2], rawdata[3], 
+								"Adding pid %d (type 0x%x) RegDesc len %d (%c%c%c%c)",
+								stream.getPid(), stream.getStreamType(),
+								d->getLength(), rawdata[2], rawdata[3],
 								rawdata[4], rawdata[5]);
 							delete d;
 							return stream.getPid();
 						}
 					}
 					break;
-				default: 
+				default:
 					break;
 				}
 				delete d;
 			}
 			if(!found) {
-				isyslog("Adding pid %d (type 0x%x) RegDesc not found -> assume AC-3", 
+				isyslog("Adding pid %d (type 0x%x) RegDesc not found -> assume AC-3",
 					stream.getPid(), stream.getStreamType());
 				return stream.getPid();
 			}
@@ -254,7 +261,12 @@ void cStreamdevPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, i
 			SI::PAT::Association assoc;
 			for (SI::Loop::Iterator it; pat.associationLoop.getNext(assoc, it); ) {
 				if (!assoc.isNITPid()) {
+#if APIVERSNUM >= 20300
+					LOCK_CHANNELS_READ;
+					const cChannel *Channel =  Channels->GetByServiceID(Source(), Transponder(), assoc.getServiceId());
+#else
 					const cChannel *Channel =  Channels.GetByServiceID(Source(), Transponder(), assoc.getServiceId());
+#endif
 					if (Channel && (Channel == m_Channel)) {
 						int prevPmtPid = pmtPid;
 						if (0 != (pmtPid = assoc.getPid())) {
@@ -322,11 +334,11 @@ void cStreamdevPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, i
 		int pids[MAXRECEIVEPIDS + 1], npids = 0;
 		pids[npids++] = pmtPid;
 #if 0
-		pids[npids++] = 0x10;  // pid 0x10, tid 0x40: NIT 
+		pids[npids++] = 0x10;  // pid 0x10, tid 0x40: NIT
 #endif
-		pids[npids++] = 0x11;  // pid 0x11, tid 0x42: SDT 
-		pids[npids++] = 0x14;  // pid 0x14, tid 0x70: TDT 
-		pids[npids++] = 0x12;  // pid 0x12, tid 0x4E...0x6F: EIT 
+		pids[npids++] = 0x11;  // pid 0x11, tid 0x42: SDT
+		pids[npids++] = 0x14;  // pid 0x14, tid 0x70: TDT
+		pids[npids++] = 0x12;  // pid 0x12, tid 0x4E...0x6F: EIT
 		for (SI::Loop::Iterator it; pmt.streamLoop.getNext(stream, it); )
 			if (0 != (pids[npids] = GetPid(stream)) && npids < MAXRECEIVEPIDS)
 				npids++;
@@ -338,32 +350,39 @@ void cStreamdevPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, i
 
 // --- cStreamdevLiveStreamer -------------------------------------------------
 
-cStreamdevLiveStreamer::cStreamdevLiveStreamer(int Priority, const cServerConnection *Connection):
+cStreamdevLiveStreamer::cStreamdevLiveStreamer(const cServerConnection *Connection, const cChannel *Channel, int Priority, eStreamType StreamType, const int* Apid, const int *Dpid) :
 		cStreamdevStreamer("streamdev-livestreaming", Connection),
 		m_Priority(Priority),
 		m_NumPids(0),
-		m_StreamType(stTSPIDS),
-		m_Channel(NULL),
+		m_Channel(Channel),
 		m_Device(NULL),
 		m_Receiver(NULL),
 		m_PatFilter(NULL),
-		m_Remux(NULL)
+		m_SwitchLive(false)
 {
+		m_ReceiveBuffer = new cStreamdevBuffer(LIVEBUFSIZE, TS_SIZE *2, true, "streamdev-livestreamer"),
+		m_ReceiveBuffer->SetTimeouts(0, 100);
+		if (Priority == IDLEPRIORITY) {
+			SetChannel(StreamType, Apid, Dpid);
+		}
+		else {
+			m_Device = SwitchDevice(Channel, Priority);
+			if (m_Device)
+				SetChannel(StreamType, Apid, Dpid);
+			memcpy(m_Caids,Channel->Caids(),sizeof(m_Caids));
+		}
 }
 
-cStreamdevLiveStreamer::~cStreamdevLiveStreamer() 
+cStreamdevLiveStreamer::~cStreamdevLiveStreamer()
 {
 	Dprintf("Desctructing Live streamer\n");
 	Stop();
-	if(m_PatFilter) {
-		Detach();
-		DELETENULL(m_PatFilter);
-	}
+	DELETENULL(m_PatFilter);
 	DELETENULL(m_Receiver);
-	delete m_Remux;
+	delete m_ReceiveBuffer;
 }
 
-bool cStreamdevLiveStreamer::HasPid(int Pid) 
+bool cStreamdevLiveStreamer::HasPid(int Pid)
 {
 	int idx;
 	for (idx = 0; idx < m_NumPids; ++idx)
@@ -372,13 +391,13 @@ bool cStreamdevLiveStreamer::HasPid(int Pid)
 	return false;
 }
 
-bool cStreamdevLiveStreamer::SetPid(int Pid, bool On) 
+bool cStreamdevLiveStreamer::SetPid(int Pid, bool On)
 {
 	int idx;
 
 	if (Pid == 0)
 		return true;
-	
+
 	if (On) {
 		for (idx = 0; idx < m_NumPids; ++idx) {
 			if (m_Pids[idx] == Pid)
@@ -440,7 +459,13 @@ bool cStreamdevLiveStreamer::SetPids(int Pid, const int *Pids1, const int *Pids2
 void cStreamdevLiveStreamer::SetPriority(int Priority)
 {
 	m_Priority = Priority;
-	StartReceiver();
+#if VDRVERSNUM >= 20104
+	cThreadLock ThreadLock(m_Device);
+	if (m_Receiver)
+		m_Receiver->SetPriority(Priority);
+	else
+#endif
+		StartReceiver();
 }
 
 void cStreamdevLiveStreamer::GetSignal(int *DevNum, int *Strength, int *Quality) const
@@ -457,16 +482,22 @@ cString cStreamdevLiveStreamer::ToText() const
 	if (m_Device && m_Channel) {
 		return cString::sprintf("DVB%-2d %3d %s", m_Device->DeviceNumber() + 1, m_Channel->Number(), m_Channel->Name());
 	}
-        return cString("");
+	return cString("");
 }
 
-void cStreamdevLiveStreamer::StartReceiver(void)
+bool cStreamdevLiveStreamer::IsReceiving(void) const
 {
-	if (m_NumPids > 0) {
+	cThreadLock ThreadLock(m_Device);
+	return m_Receiver && m_Receiver->IsAttached();
+}
+
+void cStreamdevLiveStreamer::StartReceiver(bool Force)
+{
+	if (m_NumPids > 0 || Force) {
 		Dprintf("Creating Receiver to respect changed pids\n");
 		cReceiver *current = m_Receiver;
-		m_Receiver = new cStreamdevLiveReceiver(this, m_Channel, m_Priority, m_Pids);
 		cThreadLock ThreadLock(m_Device);
+		m_Receiver = new cStreamdevLiveReceiver(this, m_Channel, m_Priority, m_Pids);
 		if (IsRunning())
 			Attach();
 		delete current;
@@ -475,38 +506,38 @@ void cStreamdevLiveStreamer::StartReceiver(void)
 		DELETENULL(m_Receiver);
 }
 
-bool cStreamdevLiveStreamer::SetChannel(const cChannel *Channel, eStreamType StreamType, const int* Apid, const int *Dpid) 
+bool cStreamdevLiveStreamer::SetChannel(eStreamType StreamType, const int* Apid, const int *Dpid)
 {
 	Dprintf("Initializing Remuxer for full channel transfer\n");
 	//printf("ca pid: %d\n", Channel->Ca());
-	m_Channel = Channel;
-	m_StreamType = StreamType;
 
 	const int *Apids = Apid ? Apid : m_Channel->Apids();
 	const int *Dpids = Dpid ? Dpid : m_Channel->Dpids();
 
-	switch (m_StreamType) {
-	case stES: 
+	switch (StreamType) {
+	case stES:
 		{
 			int pid = ISRADIO(m_Channel) ? m_Channel->Apid(0) : m_Channel->Vpid();
 			if (Apid && Apid[0])
 				pid = Apid[0];
 			else if (Dpid && Dpid[0])
 				pid = Dpid[0];
-			m_Remux = new cTS2ESRemux(pid);
+			SetRemux(new cTS2ESRemux(pid));
 			return SetPids(pid);
 		}
 
-	case stPES: 
-		m_Remux = new cTS2PESRemux(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids());
+	case stPES:
+		SetRemux(new cTS2PESRemux(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids()));
 		return SetPids(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids());
 
-	case stPS:  
-		m_Remux = new cTS2PSRemux(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids());
+#ifdef STREAMDEV_PS
+	case stPS:
+		SetRemux(new cTS2PSRemux(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids()));
 		return SetPids(m_Channel->Vpid(), Apids, Dpids, m_Channel->Spids());
+#endif
 
 	case stEXT:
-		m_Remux = new cExternRemux(Connection(), m_Channel, Apids, Dpids);
+		SetRemux(new cExternRemux(Connection(), m_Channel, Apids, Dpids));
 		// fall through
 	case stTS:
 		// This should never happen, but ...
@@ -524,76 +555,194 @@ bool cStreamdevLiveStreamer::SetChannel(const cChannel *Channel, eStreamType Str
 
 	case stTSPIDS:
 		Dprintf("pid streaming mode\n");
+		// No PIDs requested yet. Start receiver anyway to occupy device
+		StartReceiver(true);
 		return true;
 	default:
 		return false;
 	}
 }
 
-int cStreamdevLiveStreamer::Put(const uchar *Data, int Count) 
+#if APIVERSNUM >= 20300
+void cStreamdevLiveStreamer::Receive(const uchar *Data, int Length)
+#else
+void cStreamdevLiveStreamer::Receive(uchar *Data, int Length)
+#endif
+{
+	int p = m_ReceiveBuffer->PutTS(Data, Length);
+	if (p != Length)
+		m_ReceiveBuffer->ReportOverflow(Length - p);
+}
+
+void cStreamdevLiveStreamer::Action(void)
+{
+	if (StreamdevServerSetup.LiveBufferMs) {
+		// wait for first data block
+		int count = 0;
+		while (Running()) {
+			if (m_ReceiveBuffer->Get(count) != NULL) {
+				cCondWait::SleepMs(StreamdevServerSetup.LiveBufferMs);
+				break;
+			}
+		}
+	}
+	cStreamdevStreamer::Action();
+}
+
+int cStreamdevLiveStreamer::Put(const uchar *Data, int Count)
 {
 	// insert si data
 	if (m_PatFilter) {
 		int siCount;
 		uchar *siData = m_PatFilter->Get(siCount);
 		if (siData) {
-			if (m_Remux)
-				siCount = m_Remux->Put(siData, siCount);
-			else
-				siCount = cStreamdevStreamer::Put(siData, siCount);
+			siCount = cStreamdevStreamer::Put(siData, siCount);
 			if (siCount)
 				m_PatFilter->Del(siCount);
 		}
 	}
-	if (m_Remux)
-		return m_Remux->Put(Data, Count);
-	else
-		return cStreamdevStreamer::Put(Data, Count);
+	return cStreamdevStreamer::Put(Data, Count);
 }
 
-uchar *cStreamdevLiveStreamer::Get(int &Count)
+void cStreamdevLiveStreamer::Attach(void)
 {
-	if (m_Remux)
-		return m_Remux->Get(Count);
-	else
-		return cStreamdevStreamer::Get(Count);
-}
-
-void cStreamdevLiveStreamer::Del(int Count)
-{
-	if (m_Remux)
-		m_Remux->Del(Count);
-	else
-		cStreamdevStreamer::Del(Count);
-}
-	
-void cStreamdevLiveStreamer::Attach(void) 
-{ 
 	Dprintf("cStreamdevLiveStreamer::Attach()\n");
 	if (m_Device) {
 		if (m_Receiver) {
-			m_Device->Detach(m_Receiver); 
-			m_Device->AttachReceiver(m_Receiver); 
+			if (m_Receiver->IsAttached())
+				m_Device->Detach(m_Receiver);
+			m_Device->AttachReceiver(m_Receiver);
 		}
 		if (m_PatFilter) {
-			m_Device->Detach(m_PatFilter); 
-			m_Device->AttachFilter(m_PatFilter); 
+			m_Device->Detach(m_PatFilter);
+			m_Device->AttachFilter(m_PatFilter);
 		}
 	}
 }
 
-void cStreamdevLiveStreamer::Detach(void) 
-{ 
+void cStreamdevLiveStreamer::Detach(void)
+{
 	Dprintf("cStreamdevLiveStreamer::Detach()\n");
 	if (m_Device) {
 		if (m_Receiver)
-			m_Device->Detach(m_Receiver); 
+			m_Device->Detach(m_Receiver);
 		if (m_PatFilter)
-			m_Device->Detach(m_PatFilter); 
+			m_Device->Detach(m_PatFilter);
 	}
 }
 
-std::string cStreamdevLiveStreamer::Report(void) 
+bool cStreamdevLiveStreamer::UsedByLiveTV(cDevice *device)
+{
+	return device == cTransferControl::ReceiverDevice() ||
+		(device->IsPrimaryDevice() && device->HasDecoder() && !device->Replaying());
+}
+
+cDevice *cStreamdevLiveStreamer::SwitchDevice(const cChannel *Channel, int Priority)
+{
+	cDevice *device = cDevice::GetDevice(Channel, Priority, false);
+	if (!device) {
+		dsyslog("streamdev: GetDevice failed for channel %d (%s) at priority %d (PrimaryDevice=%d, ActualDevice=%d)", Channel->Number(), Channel->Name(), Priority, cDevice::PrimaryDevice()->CardIndex(), cDevice::ActualDevice()->CardIndex());
+	}
+	else if (!device->IsTunedToTransponder(Channel) && UsedByLiveTV(device)) {
+		// make sure VDR main loop doesn't switch back
+		device->SetOccupied(STREAMDEVTUNETIMEOUT);
+		if (device->SwitchChannel(Channel, false)) {
+			// switched away live TV
+			m_SwitchLive = true;
+		}
+		else {
+			dsyslog("streamdev: SwitchChannel (live) failed for channel %d (%s) at priority %d (PrimaryDevice=%d, ActualDevice=%d, device=%d)", Channel->Number(), Channel->Name(), Priority, cDevice::PrimaryDevice()->CardIndex(), cDevice::ActualDevice()->CardIndex(), device->CardIndex());
+			device->SetOccupied(0);
+			device = NULL;
+		}
+	}
+	else if (!device->SwitchChannel(Channel, false)) {
+		dsyslog("streamdev: SwitchChannel failed for channel %d (%s) at priority %d (PrimaryDevice=%d, ActualDevice=%d, device=%d)", Channel->Number(), Channel->Name(), Priority, cDevice::PrimaryDevice()->CardIndex(), cDevice::ActualDevice()->CardIndex(), device->CardIndex());
+		device = NULL;
+	}
+	return device;
+}
+
+bool cStreamdevLiveStreamer::ProvidesChannel(const cChannel *Channel, int Priority)
+{
+	cDevice *device = cDevice::GetDevice(Channel, Priority, false, true);
+	if (!device)
+		dsyslog("streamdev: No device provides channel %d (%s) at priority %d", Channel->Number(), Channel->Name(), Priority);
+	return device;
+}
+
+void cStreamdevLiveStreamer::ChannelChange(const cChannel *Channel)
+{
+	if (Running() && m_Device && m_Channel == Channel) {
+		Detach();
+		int NumUsableSlots = 0;
+		if (Channel->Ca() >= CA_ENCRYPTED_MIN) {
+			for (cCamSlot *CamSlot = CamSlots.First(); CamSlot; CamSlot = CamSlots.Next(CamSlot)) {
+				if (CamSlot->ModuleStatus() == msReady) {
+					if (CamSlot->ProvidesCa(Channel->Caids())) {
+#if APIVERSNUM >= 20303
+						if (!ChannelCamRelations.CamChecked(Channel->GetChannelID(), CamSlot->MasterSlotNumber())) {
+#else
+						if (!ChannelCamRelations.CamChecked(Channel->GetChannelID(), CamSlot->SlotNumber())) {
+#endif
+							NumUsableSlots++;
+						}
+					}
+				}
+			}
+			if (!NumUsableSlots) {
+				isyslog("streamdev: cam slot not available");
+				return;
+			}
+		}
+
+		if (m_Device = cDevice::GetDevice(Channel, LIVEPRIORITY, false)) {
+			if (m_Device->SwitchChannel(Channel, false)) {
+				Attach();
+			} else {
+				isyslog("streamdev: failed to re-tune after channel %d (%s) changed", Channel->Number(), Channel->Name());
+			}
+		} else {
+			isyslog("streamdev: device not available");
+		}
+	}
+}
+
+void cStreamdevLiveStreamer::MainThreadHook()
+{
+	if (!m_SwitchLive && Running() && m_Device && !m_Device->IsTunedToTransponder(m_Channel) && !IsReceiving()) {
+		cDevice *dev = SwitchDevice(m_Channel, m_Priority);
+		if (dev) {
+			dsyslog("streamdev: Lost channel %d (%s) on device %d. Continuing on device %d.", m_Channel->Number(), m_Channel->Name(), m_Device->CardIndex(), dev->CardIndex());
+			m_Device = dev;
+			StartReceiver();
+		}
+		else {
+			isyslog("streamdev: Lost channel %d (%s) on device %d.", m_Channel->Number(), m_Channel->Name(), m_Device->CardIndex());
+			Stop();
+		}
+	}
+	if (m_SwitchLive) {
+		// switched away live TV. Try previous channel on other device first
+#if APIVERSNUM >= 20300
+		LOCK_CHANNELS_READ;
+		if (!Channels->SwitchTo(cDevice::CurrentChannel())) {
+			// switch to streamdev channel otherwise
+			Channels->SwitchTo(m_Channel->Number());
+#else
+		if (!Channels.SwitchTo(cDevice::CurrentChannel())) {
+			// switch to streamdev channel otherwise
+			Channels.SwitchTo(m_Channel->Number());
+#endif
+			Skins.Message(mtInfo, tr("Streaming active"));
+		}
+		if (m_Device)
+			m_Device->SetOccupied(0);
+		m_SwitchLive = false;
+	}
+}
+
+std::string cStreamdevLiveStreamer::Report(void)
 {
 	std::string result;
 
@@ -601,113 +750,11 @@ std::string cStreamdevLiveStreamer::Report(void)
 		result += (std::string)"+- Device is " + (const char*)itoa(m_Device->CardIndex()) + "\n";
 	if (m_Receiver != NULL)
 		result += "+- Receiver is allocated\n";
-		
+
 	result += "+- Pids are ";
-	for (int i = 0; i < MAXRECEIVEPIDS; ++i) 
+	for (int i = 0; i < MAXRECEIVEPIDS; ++i)
 		if (m_Pids[i] != 0)
 			result += (std::string)(const char*)itoa(m_Pids[i]) + ", ";
 	result += "\n";
 	return result;
 }
-
-// --- cStreamdevFilterStreamer -------------------------------------------------
-
-cStreamdevFilterStreamer::cStreamdevFilterStreamer():
-		cStreamdevStreamer("streamdev-filterstreaming"),
-		m_Device(NULL),
-		m_Filter(NULL)/*,
-		m_Channel(NULL)*/
-{
-}
-
-cStreamdevFilterStreamer::~cStreamdevFilterStreamer() 
-{
-	Dprintf("Desctructing Filter streamer\n");
-	Detach();
-	m_Device = NULL;
-	DELETENULL(m_Filter);
-	Stop();
-}
-
-void cStreamdevFilterStreamer::Attach(void) 
-{ 
-	Dprintf("cStreamdevFilterStreamer::Attach()\n");
-	LOCK_THREAD;
-	if(m_Device && m_Filter)
-		m_Device->AttachFilter(m_Filter);
-}
-
-void cStreamdevFilterStreamer::Detach(void) 
-{ 
-	Dprintf("cStreamdevFilterStreamer::Detach()\n");
-	LOCK_THREAD;
-	if(m_Device && m_Filter)
-		m_Device->Detach(m_Filter); 
-}
-
-#if 0
-void cStreamdevFilterStreamer::SetChannel(const cChannel *Channel)
-{
-	LOCK_THREAD;
-	Dprintf("cStreamdevFilterStreamer::SetChannel(%s : %s)", Channel?Channel->Name():"<null>",
-		Channel ? *Channel->GetChannelID().ToString() : "");
-	m_Channel = Channel;
-}
-#endif
-
-void cStreamdevFilterStreamer::SetDevice(cDevice *Device)
-{
-	Dprintf("cStreamdevFilterStreamer::SetDevice()\n");
-	LOCK_THREAD;
-	Detach();
-	m_Device = Device;
-	//m_Channel = NULL;
-	Attach();
-}
-
-bool cStreamdevFilterStreamer::SetFilter(u_short Pid, u_char Tid, u_char Mask, bool On) 
-{	
-	Dprintf("cStreamdevFilterStreamer::SetFilter(%u,0x%x,0x%x,%s)\n", Pid, Tid, Mask, On?"On":"Off");
-
-	if(!m_Device)
-		return false;
-
-	if (On) {
-		if (m_Filter == NULL) {
-			m_Filter = new cStreamdevLiveFilter(this);
-			Dprintf("attaching filter to device\n");
-			Attach();
-		}
-		m_Filter->Set(Pid, Tid, Mask);
-	} else if (m_Filter != NULL) 
-		m_Filter->Del(Pid, Tid, Mask);
-
-	return true;
-}
-
-#if 0
-void cStreamdevFilterStreamer::ChannelSwitch(const cDevice *Device, int ChannelNumber) {
-	LOCK_THREAD;
-	if(Device == m_Device) {
-		if(ChannelNumber > 0) {
-			cChannel *ch = Channels.GetByNumber(ChannelNumber);
-			if(ch != NULL) {
-				if(m_Filter != NULL &&
-						m_Channel != NULL &&
-						(! TRANSPONDER(ch, m_Channel))) {
-
-					isyslog("***** LiveFilterStreamer: transponder changed ! %s",
-						*ch->GetChannelID().ToString());
-
-					uchar buffer[TS_SIZE] = {TS_SYNC_BYTE, 0xff, 0xff, 0xff, 0x7f, 0};
-					strcpy((char*)(buffer + 5), ch->GetChannelID().ToString());
-					int p = Put(buffer, TS_SIZE);
-					if (p != TS_SIZE)
-						ReportOverflow(TS_SIZE - p);
-				}
-				m_Channel = ch;
-			}
-		}
-	}
-}
-#endif
