@@ -1,5 +1,5 @@
 /*
- *  $Id: setup.h,v 1.4 2010/07/19 13:49:31 schmirl Exp $
+ *  setup.h
  */
  
 #ifndef VDR_STREAMDEV_SETUPSERVER_H
@@ -7,6 +7,13 @@
 
 #include "common.h"
 
+enum eStartSuspended {
+	ssNo,
+	ssYes,
+	ssAuto,
+	ss_Count
+};
+
 struct cStreamdevServerSetup {
 	cStreamdevServerSetup(void);
 
@@ -14,6 +21,8 @@ struct cStreamdevServerSetup {
 
 	int HideMenuEntry;
 	int MaxClients;
+	int StartSuspended;
+	int LiveBufferMs;
 	int StartVTPServer;
 	int VTPServerPort;
 	char VTPBindIP[20];
